# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: install.py 1229 2011-01-29 17:07:16Z icemac $
"""Initial generation."""


def evolve(context):
    # nothing to install initially
    pass
