# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: __init__.py 1356 2011-10-28 20:00:27Z icemac $


class Search(object):
    """View to select a search."""
    show_headline = True
    form_explanation = u''
