# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: i18n.py 1229 2011-01-29 17:07:16Z icemac $

"""I18n message factory for icemac.addressbook. """

import zope.i18nmessageid

_ = MessageFactory = zope.i18nmessageid.MessageFactory('icemac.addressbook')
